package it.tref.liferay.mapit.map.portlet;

import it.tref.liferay.mapit.constants.Constants;
import it.tref.liferay.mapit.constants.PortletConfiguration;
import it.tref.liferay.mapit.constants.WebKeys;
import it.tref.liferay.mapit.map.model.Plugin;
import it.tref.liferay.mapit.map.service.PluginLocalServiceUtil;
import it.tref.liferay.mapit.map.util.MapItPluginPortletPreferencesUtil;
import it.tref.liferay.mapit.map.util.MapItPluginUtil;
import it.tref.liferay.mapit.plugin.MapItPlugin;
import it.tref.liferay.mapit.plugin.action.MapItPluginAction;
import it.tref.liferay.mapit.util.MapFeatureUtil;
import it.tref.liferay.mapit.util.SearchUtil;

import java.io.IOException;
import java.util.Set;

import javax.portlet.PortletException;
import javax.portlet.PortletPreferences;
import javax.portlet.ResourceRequest;
import javax.portlet.ResourceResponse;
import javax.servlet.RequestDispatcher;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.search.Hits;
import com.liferay.portal.kernel.servlet.DirectRequestDispatcherFactoryUtil;
import com.liferay.portal.kernel.servlet.PipingServletResponse;
import com.liferay.portal.kernel.servlet.ServletContextPool;
import com.liferay.portal.kernel.util.ArrayUtil;
import com.liferay.portal.kernel.util.ClassLoaderPool;
import com.liferay.portal.kernel.util.ClassResolverUtil;
import com.liferay.portal.kernel.util.ContentTypes;
import com.liferay.portal.kernel.util.ParamUtil;
import com.liferay.portal.kernel.util.ReflectionUtil;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.kernel.util.StringUtil;
import com.liferay.portal.kernel.util.Validator;
import com.liferay.portal.service.ServiceContext;
import com.liferay.portal.service.ServiceContextFactory;
import com.liferay.portal.theme.ThemeDisplay;
import com.liferay.portal.util.PortalUtil;
import com.liferay.util.bridges.mvc.MVCPortlet;

public class MapItPortlet extends MVCPortlet {

  @Override
  public void serveResource(ResourceRequest resourceRequest, ResourceResponse resourceResponse) throws IOException,
      PortletException {

    String resourceId = resourceRequest.getResourceID();
    String contextName = ParamUtil.getString(resourceRequest, "contextName");

    if (Validator.isNull(contextName)) {
      if (resourceId.equals("assetEntries")) {
        try {
          writeFeatures(resourceRequest, resourceResponse);
        } catch (Exception e) {
          _log.error(e, e);
          throw new PortletException(e);
        }
      } else {
        super.serveResource(resourceRequest, resourceResponse);
      }
    } else {
      try {
        if (!executeMapItPluginAction(resourceId, contextName, resourceRequest, resourceResponse)) {
          includeMapItPlugin(contextName, resourceRequest, resourceResponse);
        }
      } catch (Exception e) {
        _log.error(e, e);
        throw new PortletException(e);
      }
    }
  }

  private void writeFeatures(ResourceRequest request, ResourceResponse response) throws Exception {

    ServiceContext serviceContext = ServiceContextFactory.getInstance(request);

    long companyId = serviceContext.getCompanyId();
    long groupId = serviceContext.getScopeGroupId();

    long classNameId = ParamUtil.getLong(request, "classNameId");

    HttpServletRequest originalServletRequest =
        PortalUtil.getOriginalServletRequest(PortalUtil.getHttpServletRequest(request));

    String bbox = ParamUtil.getString(originalServletRequest, "bbox");

    double[] bboxSplit = null;
    if (Validator.isNotNull(bbox)) {
      bboxSplit = StringUtil.split(bbox, 0d);
    }

    Hits hits =
        SearchUtil.search(companyId, groupId, classNameId, null, 0, Constants.MAX_SEARCH_RESULT, null, bboxSplit);

    String output = MapFeatureUtil.toJSON(hits, request, response);

    response.setContentType(ContentTypes.APPLICATION_JSON);
    response.getPortletOutputStream().write(output.getBytes());
    response.getPortletOutputStream().close();
  }

  /*
   * MapIt Plugin
   */

  private boolean executeMapItPluginAction(String resourceId, String contextName, ResourceRequest resourceRequest,
      ResourceResponse resourceResponse) throws Exception {

    PortletPreferences portletPreferences = resourceRequest.getPreferences();

    String[] pluginIds =
        StringUtil.split(portletPreferences.getValue(PortletConfiguration.PLUGIN_IDS, StringPool.BLANK));

    Plugin plugin = PluginLocalServiceUtil.getPlugin(contextName);

    if (plugin != null && plugin.isActive() && ArrayUtil.contains(pluginIds, contextName)) {

      MapItPlugin mapItPlugin = MapItPluginUtil.getMapItPlugin(plugin.getContext());

      Set<Class<? extends MapItPluginAction>> mapItPluginActions = mapItPlugin.getMapItPluginActions();

      if (mapItPluginActions != null) {

        for (Class<? extends MapItPluginAction> action : mapItPluginActions) {

          ClassLoader contextClassLoader = ClassLoaderPool.getClassLoader(plugin.getContext());
          Class<?> clazz = ClassResolverUtil.resolve(action.getName(), contextClassLoader);
          Object obj = clazz.getConstructor(new Class[0]).newInstance(new Object[0]);

          String id = (String) ReflectionUtil.getDeclaredMethod(clazz, "getResourceId").invoke(obj);

          if (resourceId.equals(id)) {

            PortletPreferences preferences = null;

            if (Validator.isNotNull(mapItPlugin.getPortletConfigurationJsp())) {
              ThemeDisplay themeDisplay = (ThemeDisplay) resourceRequest.getAttribute(WebKeys.THEME_DISPLAY);
              preferences =
                  MapItPluginPortletPreferencesUtil.getPortletPreferences(themeDisplay.getCompanyId(),
                      themeDisplay.getPlid(), themeDisplay.getPortletDisplay().getId(), contextName);
            }

            ReflectionUtil.getDeclaredMethod(clazz, "execute", ResourceRequest.class, ResourceResponse.class,
                PortletPreferences.class).invoke(obj, resourceRequest, resourceResponse, preferences);

            return true;
          }
        }
      }
    }

    return false;
  }

  private void includeMapItPlugin(String contextName, ResourceRequest resourceRequest, ResourceResponse resourceResponse)
      throws IOException, ServletException {

    ServletContext servletContext = ServletContextPool.get(contextName);

    String path = getPath(resourceRequest);

    if (Validator.isNotNull(path)) {

      RequestDispatcher requestDispatcher =
          DirectRequestDispatcherFactoryUtil.getRequestDispatcher(servletContext, path);

      HttpServletRequest request = PortalUtil.getHttpServletRequest(resourceRequest);

      request.setAttribute(WebKeys.SERVLET_CONTEXT_INCLUDE_FILTER_STRICT, false);

      HttpServletResponse response =
          new PipingServletResponse(PortalUtil.getHttpServletResponse(resourceResponse),
              resourceResponse.getPortletOutputStream());

      requestDispatcher.include(request, response);

      request.removeAttribute(WebKeys.SERVLET_CONTEXT_INCLUDE_FILTER_STRICT);
    }
  }

  private final static Log _log = LogFactoryUtil.getLog(MapItPortlet.class);

}
